@@ -1,7 +1,7 @@
 package SQL::Translator::Parser::PostgreSQL;
 
 # -------------------------------------------------------------------
-# $Id: PostgreSQL.pm,v 1.43 2004/10/23 20:18:44 cmungall Exp $
+# $Id: PostgreSQL.pm,v 1.50 2007-11-13 19:25:04 mwz444 Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -108,7 +108,7 @@ View table:
 
 use strict;
 use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.43 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.50 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
@@ -154,6 +154,9 @@ statement : create
   | connect
   | update
   | set
+  | select
+  | copy
+  | readin_symbol
   | <error>
 
 connect : /^\s*\\\connect.*\n/
@@ -162,7 +165,9 @@ set : /set/i /[^;]*/ ';'
 
 revoke : /revoke/i WORD(s /,/) /on/i TABLE(?) table_name /from/i name_with_opt_quotes(s /,/) ';'
     {
-        my $table_name = $item{'table_name'};
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         push @{ $tables{ $table_name }{'permissions'} }, {
             type       => 'revoke',
             actions    => $item[2],
@@ -170,9 +175,14 @@ revoke : /revoke/i WORD(s /,/) /on/i TABLE(?) table_name /from/i name_with_opt_q
         }
     }
 
+revoke : /revoke/i WORD(s /,/) /on/i SCHEMA(?) schema_name /from/i name_with_opt_quotes(s /,/) ';'
+    { 1 }
+
 grant : /grant/i WORD(s /,/) /on/i TABLE(?) table_name /to/i name_with_opt_quotes(s /,/) ';'
     {
-        my $table_name = $item{'table_name'};
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         push @{ $tables{ $table_name }{'permissions'} }, {
             type       => 'grant',
             actions    => $item[2],
@@ -180,20 +190,33 @@ grant : /grant/i WORD(s /,/) /on/i TABLE(?) table_name /to/i name_with_opt_quote
         }
     }
 
+grant : /grant/i WORD(s /,/) /on/i SCHEMA(?) schema_name /to/i name_with_opt_quotes(s /,/) ';'
+    { 1 }
+
 drop : /drop/i /[^;]*/ ';'
 
-insert : /insert/i /[^;]*/ ';'
+string :
+   /'(\\.|''|[^\\\'])*'/ 
 
-update : /update/i /[^;]*/ ';'
+nonstring : /[^;\'"]+/
+
+statement_body : (string | nonstring)(s?)
+
+insert : /insert/i statement_body ';'
+
+update : /update/i statement_body ';'
 
 #
 # Create table.
 #
-create : create_table table_name '(' create_definition(s /,/) ')' table_option(s?) ';'
+create : create_table table_name '(' create_definition(s? /,/) ')' table_option(s?) ';'
     {
-        my $table_name                       = $item{'table_name'};
-        $tables{ $table_name }{'order'}      = ++$table_order;
-        $tables{ $table_name }{'table_name'} = $table_name;
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        $tables{ $table_name }{'order'}       = ++$table_order;
+        $tables{ $table_name }{'schema_name'} = $schema_name;
+        $tables{ $table_name }{'table_name'}  = $table_name;
 
         if ( @table_comments ) {
             $tables{ $table_name }{'comments'} = [ @table_comments ];
@@ -231,7 +254,10 @@ create : create_table table_name '(' create_definition(s /,/) ')' table_option(s
 
 create : CREATE unique(?) /(index|key)/i index_name /on/i table_name using_method(?) '(' field_name(s /,/) ')' where_predicate(?) ';'
     {
-        push @{ $tables{ $item{'table_name'} }{'indices'} },
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        push @{ $tables{ $table_name }{'indices'} },
             {
                 name      => $item{'index_name'},
                 supertype => $item{'unique'}[0] ? 'constraint' : 'index',
@@ -268,7 +294,10 @@ comment : /^\s*(?:#|-{2})(.*)\n/
 
 comment_on_table : /comment/i /on/i /table/i table_name /is/i comment_phrase ';'
     {
-        push @{ $tables{ $item{'table_name'} }{'comments'} }, $item{'comment_phrase'};
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        push @{ $tables{ $table_name }{'comments'} }, $item{'comment_phrase'};
     }
 
 comment_on_column : /comment/i /on/i /column/i column_name /is/i comment_phrase ';'
@@ -394,8 +423,8 @@ column_constraint : constraint_name(?) column_constraint_type deferrable(?) defe
             reference_table  => $desc->{'reference_table'},
             reference_fields => $desc->{'reference_fields'},
             match_type       => $desc->{'match_type'},
-            on_delete_do     => $desc->{'on_delete_do'},
-            on_update_do     => $desc->{'on_update_do'},
+            on_delete        => $desc->{'on_delete'} || $desc->{'on_delete_do'},
+            on_update        => $desc->{'on_update'} || $desc->{'on_update_do'},
         } 
     }
 
@@ -417,6 +446,9 @@ column_constraint_type : /not null/i { $return = { type => 'not_null' } }
     |
     /references/i table_name parens_word_list(?) match_type(?) key_action(s?)
     {
+        my $table_info  = $item{'table_name'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         my ( $on_delete, $on_update );
         for my $action ( @{ $item[5] || [] } ) {
             $on_delete = $action->{'action'} if $action->{'type'} eq 'delete';
@@ -425,15 +457,21 @@ column_constraint_type : /not null/i { $return = { type => 'not_null' } }
 
         $return              =  {
             type             => 'foreign_key',
-            reference_table  => $item[2],
+            reference_table  => $table_name,
             reference_fields => $item[3][0],
             match_type       => $item[4][0],
-            on_delete_do     => $on_delete,
-            on_update_do     => $on_update,
+            on_delete        => $on_delete,
+            on_update        => $on_update,
         }
     }
 
-table_name : name_with_opt_quotes
+table_name : schema_qualification(?) name_with_opt_quotes {
+    $return = { schema_name => $item[1], table_name => $item[2] }
+}
+
+  schema_qualification : name_with_opt_quotes '.'
+
+schema_name : name_with_opt_quotes
 
 field_name : name_with_opt_quotes
 
@@ -546,12 +584,7 @@ pg_data_type :
             $return = { type => 'bytea' };
         }
     |
-    /(timestamptz|timestamp)( with time zone)?/i
-        { 
-            $return = { type => 'timestamp' };
-        }
-    |
-    /(timestamptz|timestamp)( without time zone)?/i
+    /(timestamptz|timestamp)(?:\(\d\))?( with(out)? time zone)?/i
         { 
             $return = { type => 'timestamp' };
         }
@@ -601,8 +634,8 @@ table_constraint : comment(s?) constraint_name(?) table_constraint_type deferrab
             reference_table  => $desc->{'reference_table'},
             reference_fields => $desc->{'reference_fields'},
             match_type       => $desc->{'match_type'}[0],
-            on_delete_do     => $desc->{'on_delete_do'},
-            on_update_do     => $desc->{'on_update_do'},
+            on_delete        => $desc->{'on_delete'} || $desc->{'on_delete_do'},
+            on_update        => $desc->{'on_update'} || $desc->{'on_update_do'},
             comments         => [ @comments ],
         } 
     }
@@ -643,15 +676,15 @@ table_constraint_type : /primary key/i '(' name_with_opt_quotes(s /,/) ')'
             supertype        => 'constraint',
             type             => 'foreign_key',
             fields           => $item[3],
-            reference_table  => $item[6],
+            reference_table  => $item[6]->{'table_name'},
             reference_fields => $item[7][0],
             match_type       => $item[8][0],
-            on_delete_do     => $on_delete || '',
-            on_update_do     => $on_update || '',
+            on_delete     => $on_delete || '',
+            on_update     => $on_update || '',
         }
     }
 
-deferrable : /not/i /deferrable/i 
+deferrable : not(?) /deferrable/i 
     { 
         $return = ( $item[1] =~ /not/i ) ? 0 : 1;
     }
@@ -695,7 +728,7 @@ key_mutation : /no action/i { $return = 'no_action' }
 alter : alter_table table_name add_column field ';' 
     { 
         my $field_def = $item[4];
-        $tables{ $item[2] }{'fields'}{ $field_def->{'name'} } = {
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $field_def->{'name'} } = {
             %$field_def, order => $field_order++
         };
         1;
@@ -703,7 +736,7 @@ alter : alter_table table_name add_column field ';'
 
 alter : alter_table table_name ADD table_constraint ';' 
     { 
-        my $table_name = $item[2];
+        my $table_name = $item[2]->{'table_name'};
         my $constraint = $item[4];
         push @{ $tables{ $table_name }{'constraints'} }, $constraint;
         1;
@@ -711,13 +744,13 @@ alter : alter_table table_name ADD table_constraint ';'
 
 alter : alter_table table_name drop_column NAME restrict_or_cascade(?) ';' 
     {
-        $tables{ $item[2] }{'fields'}{ $item[4] }{'drop'} = 1;
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $item[4] }{'drop'} = 1;
         1;
     }
 
 alter : alter_table table_name alter_column NAME alter_default_val ';' 
     {
-        $tables{ $item[2] }{'fields'}{ $item[4] }{'default'} = 
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $item[4] }{'default'} = 
             $item[5]->{'value'};
         1;
     }
@@ -743,6 +776,9 @@ alter : alter_table table_name DROP /constraint/i NAME restrict_or_cascade ';'
 alter : alter_table table_name /owner/i /to/i NAME ';'
     { 1 }
 
+alter : alter_sequence NAME /owned/i /by/i column_name ';'
+    { 1 }
+
 storage_type : /(plain|external|extended|main)/i
 
 alter_default_val : SET default_val 
@@ -763,7 +799,7 @@ alter_default_val : SET default_val
 #
 alter : alter_table table_name alter_column NAME alter_nullable ';'
     {
-#        my $table_name  = $item[2];
+#        my $table_name  = $item[2]->{'table_name'};
 #        my $field_name  = $item[4];
 #        my $is_nullable = $item[5]->{'is_nullable'};
 #
@@ -804,10 +840,14 @@ alter_nullable : SET not_null
 
 not_null : /not/i /null/i
 
+not : /not/i
+
 add_column : ADD COLUMN(?)
 
 alter_table : ALTER TABLE ONLY(?)
 
+alter_sequence : ALTER SEQUENCE 
+
 drop_column : DROP COLUMN(?)
 
 alter_column : ALTER COLUMN(?)
@@ -817,6 +857,24 @@ rename_column : /rename/i COLUMN(?)
 restrict_or_cascade : /restrict/i | 
     /cascade/i
 
+# Handle functions that can be called
+select : SELECT select_function ';' 
+    { 1 }
+
+# Read the setval function but don't do anything with it because this parser
+# isn't handling sequences
+select_function : schema_qualification(?) /setval/i '(' VALUE /,/ VALUE /,/ /(true|false)/i ')' 
+    { 1 }
+
+# Skipping all COPY commands
+copy : COPY WORD /[^;]+/ ';' { 1 }
+    { 1 }
+
+# The "\." allows reading in from STDIN but this isn't needed for schema
+# creation, so it is skipped.
+readin_symbol : '\.'
+    {1}
+
 #
 # End basically useless stuff. - ky
 #
@@ -877,8 +935,16 @@ COLUMN : /column/i
 
 TABLE : /table/i
 
+SCHEMA : /schema/i
+
 SEMICOLON : /\s*;\n?/
 
+SEQUENCE : /sequence/i
+
+SELECT : /select/i
+
+COPY : /copy/i
+
 INTEGER : /\d+/
 
 WORD : /\w+/
@@ -930,7 +996,8 @@ sub parse {
     for my $table_name ( @tables ) {
         my $tdata =  $result->{ $table_name };
         my $table =  $schema->add_table( 
-            name  => $tdata->{'table_name'},
+            #schema => $tdata->{'schema_name'},
+            name   => $tdata->{'table_name'},
         ) or die "Couldn't create table '$table_name': " . $schema->error;
 
         $table->comments( $tdata->{'comments'} );
@@ -979,8 +1046,8 @@ sub parse {
                 reference_table  => $cdata->{'reference_table'},
                 reference_fields => $cdata->{'reference_fields'},
                 match_type       => $cdata->{'match_type'} || '',
-                on_delete        => $cdata->{'on_delete_do'},
-                on_update        => $cdata->{'on_update_do'},
+                on_delete        => $cdata->{'on_delete'} || $cdata->{'on_delete_do'},
+                on_update        => $cdata->{'on_update'} || $cdata->{'on_update_do'},
                 expression       => $cdata->{'expression'},
             ) or die "Can't add constraint of type '" .
                 $cdata->{'type'} .  "' to table '" . $table->name . 
